@@ -1,14 +1,14 @@
 #' Model evalbin
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
 #'
 #' @param dataset Dataset name (string). This can be a dataframe in the global environment or an element in an r_data list from Radiant
 #' @param pred Predictions or predictors
 #' @param rvar Response variable
 #' @param lev The level in the response variable defined as _success_
 #' @param qnt Number of bins to create
-#' @param margin Margin on each customer purchase
 #' @param cost Cost for each connection (e.g., email or mailing)
+#' @param margin Margin on each customer purchase
 #' @param train Use data from training ("Training"), validation ("Validation"), both ("Both"), or all data ("All") to evaluate model evalbin
 #' @param method Use either ntile or xtile to split the data (default is xtile)
 #' @param data_filter Expression entered in, e.g., Data > View to filter the dataset in Radiant. The expression should be a string (e.g., "price > 10000")
@@ -25,15 +25,18 @@
 evalbin <- function(dataset, pred, rvar,
                     lev = "",
                     qnt = 10,
-                    margin = 1,
                     cost = 1,
+                    margin = 2,
                     train = "",
                     method = "xtile",
                     data_filter = "") {
 
 	## in case no inputs were provided
-	if (is.na(margin)) margin <- 0
 	if (is.na(cost)) cost <- 0
+	if (is.na(margin)) margin <- 0
+
+  if (!train %in% c("","All") && is_empty(data_filter))
+    return("** Filter required. To set a filter go to Data > View and click\n   the filter checkbox **" %>% add_class("evalbin"))
 
 	## to avoid 'global not defined' warnings
 	nr_resp <- nr_obs <- cum_resp <- cum_resp_rate <- everything <- NULL
@@ -54,7 +57,7 @@ evalbin <- function(dataset, pred, rvar,
 		dat_list[["All"]] <- getdata(dataset, vars, filt = "")
 	}
 
-	if (!is_string(dataset)) dataset <- "-----"
+	if (!is_string(dataset)) dataset <- deparse(substitute(dataset)) %>% set_attr("df", TRUE)
 
   qnt_name <- "bins"
   if (method == "xtile") method <- "radiant.data::xtile"
@@ -78,7 +81,7 @@ evalbin <- function(dataset, pred, rvar,
 	  if (lev == "") {
 	  	lev <- levs[1]
 	  } else {
-	  	if (!lev %in% levs) return(add_class("", "evalbin"))
+	  	if (!lev %in% levs) return(add_class("Level provided not found", "evalbin"))
 	  }
 
 	  ## transformation to TRUE/FALSE depending on the selected level (lev)
@@ -92,7 +95,7 @@ evalbin <- function(dataset, pred, rvar,
 
 	  for (j in seq_along(pred)) {
 	  	pname <- paste0(pred[j], pext[i])
-	  	auc_list[[pname]] <- auc(dat[[pred[j]]],dat[[rvar]], TRUE)[["W"]]
+	  	auc_list[[pname]] <- auc(dat[[pred[j]]],dat[[rvar]], TRUE)
 	  	lg_list[[pname]] <-
 			  dat %>%
 			  select_(.dots = c(pred[j],rvar)) %>%
@@ -107,7 +110,7 @@ evalbin <- function(dataset, pred, rvar,
 			    resp_rate = nr_resp / nr_obs,
 			    gains = nr_resp / tot_resp
 			  ) %>%
-			  { if (first(.$resp_rate) < last(.$resp_rate)) mutate_each(., funs(rev))
+			  { if (first(.$resp_rate) < last(.$resp_rate)) mutate_all(., funs(rev))
 			  	else . } %>%
 			  mutate(
 			    profit = margin * cumsum(nr_resp) - cost * cumsum(nr_obs),
@@ -125,9 +128,11 @@ evalbin <- function(dataset, pred, rvar,
 	  	  pl <- c(pl, max(lg_list[[pname]]$profit))
 		}
 		prof_list <- c(prof_list, pl / abs(max(pl)))
-		pdat[[i]] <- bind_rows(lg_list) %>% mutate(profit = profit / abs(max(profit)))
+		pdat[[i]] <- bind_rows(lg_list) %>% mutate(profit = profit)
 	}
 	dat <- bind_rows(pdat) %>% mutate(profit = ifelse (is.na(profit), 0, profit))
+	dat$pred <- factor(dat$pred, levels = unique(dat$pred))
+
 	names(prof_list) <- names(auc_list)
 	rm(lg_list, pdat)
 
@@ -136,10 +141,10 @@ evalbin <- function(dataset, pred, rvar,
 
 #' Summary method for the evalbin function
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
 #'
 #' @param object Return value from \code{\link{evalbin}}
-#' @param prn Print model evalbin results (default is TRUE)
+#' @param prn Print full table of measures per model and bin
 #' @param ... further arguments passed to or from other methods
 #'
 #' @seealso \code{\link{evalbin}} to summarize results
@@ -154,60 +159,61 @@ summary.evalbin <- function(object, prn = TRUE, ...) {
 
   if (is.character(object)) return(object)
 
-	if (prn) {
-		cat("Evaluate predictions for binary response models\n")
-		cat("Data        :", object$dataset, "\n")
-		if (object$data_filter %>% gsub("\\s","",.) != "")
-			cat("Filter      :", gsub("\\n","", object$data_filter), "\n")
-		cat("Results for :", object$train, "\n")
-		cat("Perdictors  :", paste0(object$pred, collapse=", "), "\n")
-		cat("Response    :", object$rvar, "\n")
-	  cat("Level       :", object$lev, "in", object$rvar, "\n")
-		cat("Bins        :", object$qnt, "\n")
-		cat("Margin:Cost :", object$margin, ":", object$cost, "\n")
-		prof <- object$prof_list
-		cat("Profit index:", paste0(names(prof), " (", round(prof,3), ")", collapse=", "), "\n")
-		auc <- unlist(object$auc_list)
-		cat("AUC         :", paste0(names(auc), " (", round(auc,3), ")", collapse=", "), "\n\n")
+	cat("Evaluate predictions for binary response models\n")
+	cat("Data        :", object$dataset, "\n")
+	if (object$data_filter %>% gsub("\\s","",.) != "")
+		cat("Filter      :", gsub("\\n","", object$data_filter), "\n")
+	cat("Results for :", object$train, "\n")
+	cat("Predictors  :", paste0(object$pred, collapse=", "), "\n")
+	cat("Response    :", object$rvar, "\n")
+  cat("Level       :", object$lev, "in", object$rvar, "\n")
+	cat("Bins        :", object$qnt, "\n")
+	cat("Cost:Margin :", object$cost, ":", object$margin, "\n")
+	# prof <- object$prof_list
+	# cat("Profit index:", paste0(names(prof), " (", round(prof,3), ")", collapse=", "), "\n")
+	# auc <- unlist(object$auc_list)
+	# cat("AUC         :", paste0(names(auc), " (", round(auc,3), ")", collapse=", "), "\n\n")
+
+	if (prn)
 		print(formatdf(as.data.frame(object$dat), 3), row.names = FALSE)
-	} else {
-    return(add_class(object$dat, "evalbin"))
-	}
 }
 
 #' Confusion matrix
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
 #'
 #' @param dataset Dataset name (string). This can be a dataframe in the global environment or an element in an r_data list from Radiant
 #' @param pred Predictions or predictors
 #' @param rvar Response variable
 #' @param lev The level in the response variable defined as _success_
-#' @param margin Margin on each customer purchase
 #' @param cost Cost for each connection (e.g., email or mailing)
+#' @param margin Margin on each customer purchase
 #' @param train Use data from training ("Training"), validation ("Validation"), both ("Both"), or all data ("All") to evaluate model evalbin
-#' @param method Use either ntile or xtile to split the data (default is xtile)
 #' @param data_filter Expression entered in, e.g., Data > View to filter the dataset in Radiant. The expression should be a string (e.g., "price > 10000")
 #' @param ... further arguments passed to or from other methods
 #'
 #' @return A list of results
 #'
-#' @seealso \code{\link{summary.evalbin}} to summarize results
-#' @seealso \code{\link{plot.evalbin}} to plot results
+#' @seealso \code{\link{summary.confusion}} to summarize results
+#' @seealso \code{\link{plot.confusion}} to plot results
 #'
-#' @examples
-#' result <- evalbin("titanic", c("age","fare"), "survived")
+#' @importFrom psych cohen.kappa
 #'
 #' @export
 confusion <- function(dataset, pred, rvar,
                       lev = "",
-                      margin = 1,
                       cost = 1,
+                      margin = 2,
                       train = "",
-                      method = "xtile",
                       data_filter = "",
                       ...) {
 
+
+  if (!train %in% c("","All") && is_empty(data_filter))
+    return("** Filter required. To set a filter go to Data > View and click the filter checkbox **" %>% add_class("confusion"))
+
+	profit <- NULL
+
 	## in case no inputs were provided
 	if (is_not(margin) || is_not(cost)) {
 		break_even <- 0.5
@@ -230,7 +236,7 @@ confusion <- function(dataset, pred, rvar,
 		dat_list[["All"]] <- getdata(dataset, vars, filt = "")
 	}
 
-	if (!is_string(dataset)) dataset <- "-----"
+	if (!is_string(dataset)) dataset <- deparse(substitute(dataset)) %>% set_attr("df", TRUE)
 
 	pdat <- list()
 	for (i in names(dat_list)) {
@@ -242,101 +248,154 @@ confusion <- function(dataset, pred, rvar,
 	      lev <- levels(rv)[1]
 	    else
 	      lev <- as.character(rv) %>% as.factor %>% levels %>% .[1]
+	  } else {
+	  	if (!lev %in% dat[[rvar]]) return(add_class("Please update the selected level in the response variable", "confusion"))
 	  }
 
-
 	  ## transformation to TRUE/FALSE depending on the selected level (lev)
 	  dat[[rvar]] <- dat[[rvar]] == lev
+
+  	auc_vec <- rep(NA, length(pred)) %>% set_names(pred)
+  	for (p in pred) auc_vec[p] <- auc(dat[[p]], dat[[rvar]], TRUE)
+
+  	p_vec <- colMeans(dat[,pred, drop = FALSE]) / mean(dat[[rvar]])
+
   	dat[, pred] <- select_(dat, .dots = pred) > break_even
 
   	if (length(pred) > 1) {
-   	  dat <- mutate_each_(dat, funs(factor(., levels = c("FALSE","TRUE"))), vars = c(rvar, pred))
+   	  dat <- mutate_at(dat, .cols = c(rvar, pred), .funs = funs(factor(., levels = c("FALSE","TRUE"))))
  		} else {
   	  dat[,pred] %<>% apply(2, function(x) factor(x, levels = c("FALSE","TRUE")))
  		}
 
 	  make_tab <- function(x) {
-	  	table(dat[[rvar]], x) %>%
-	  	  as.data.frame %>%
-	  	  .$Freq %>%
-	  	  set_names(c("TN","FN","FP","TP"))
+	  	ret <- rep(0L, 4) %>% set_names(c("TN","FN","FP","TP"))
+	  	tab <- table(dat[[rvar]], x) %>% as.data.frame
+	  	## ensure a value is availble for all four options
+	  	for (i in 1:nrow(tab)) {
+	  		if (tab[i,1] == "TRUE") {
+	  			if (tab[i,2] == "TRUE") 
+	  				ret["TP"] <- tab[i,3]
+	  			else 
+	  				ret["FN"] <- tab[i,3]
+
+	  		} else {
+	  			if (tab[i,2] == "TRUE") 
+	  				ret["FP"] <- tab[i,3]
+	  			else 
+	  				ret["TN"] <- tab[i,3]
+	  		}
+	  	}
+	  	return(ret)
 	  }
 	  ret <- lapply(select_(dat,.dots = pred), make_tab) %>% as.data.frame %>% t %>% as.data.frame
-	  ret <- bind_cols(data.frame(Type = rep(i, length(pred)), Predictor = pred), ret)
+	  ret <- bind_cols(data.frame(Type = rep(i, length(pred)), Predictor = pred), ret,
+	                   data.frame(AUC = auc_vec, p.ratio = p_vec))
+
 	  pdat[[i]] <- ret
   }
 
 	dat <- bind_rows(pdat) %>% as.data.frame %>%
-	  mutate(total = TN+FN+FP+TP, TPR = TP/(TP+FN), TNR = TN/(TN+FP))
+	  mutate(total = TN+FN+FP+TP, TPR = TP/(TP+FN), TNR = TN/(TN+FP),
+	         precision = TP / (TP + FP),
+	         accuracy = (TP + TN) / total,
+	         profit = margin * TP - cost * (TP + FP),
+	         ROME = profit / (cost * (TP + FP)),
+	         contact = (TP + FP) / total,
+	         kappa = 0)
+
+	dat <- group_by_(dat, "Type") %>% mutate(index = profit / max(profit)) %>% ungroup
+	dat <- mutate(dat, profit = as.integer(round(profit,0)))
+
+	for (i in 1:nrow(dat)) {
+		tmp <- dat[i,]
+  	dat$kappa[i] <- psych::cohen.kappa(matrix(with(tmp, c(TN,FP,FN,TP)), ncol = 2))[["kappa"]]
+	}
+
+	dat <- select_(dat, .dots = c("Type","Predictor", "TP", "FP", "TN", "FN",
+	                              "total", "TPR", "TNR", "precision", "accuracy",
+	                              "kappa", "profit", "index", "ROME", "contact", "AUC"))
+
 	rm(pdat, dat_list)
 
 	as.list(environment()) %>% add_class("confusion")
 }
 
-summary.confusion <- function(object, prn = TRUE, ...) {
+#' Summary method for the confusion matrix
+#'
+#' @details See \url{https://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
+#'
+#' @param object Return value from \code{\link{confusion}}
+#' @param ... further arguments passed to or from other methods
+#'
+#' @seealso \code{\link{confusion}} to generate results
+#' @seealso \code{\link{plot.confusion}} to visualize result
+#'
+#' @export
+summary.confusion <- function(object, ...) {
 
   if (is.character(object)) return(object)
 
-	if (prn) {
-		cat("Confusion matrix\n")
-		cat("Data        :", object$dataset, "\n")
-		if (object$data_filter %>% gsub("\\s","",.) != "")
-			cat("Filter      :", gsub("\\n","", object$data_filter), "\n")
-		cat("Results for :", object$train, "\n")
-		cat("Predictors  :", paste0(object$pred, collapse=", "), "\n")
-		cat("Response    :", object$rvar, "\n")
-	  cat("Level       :", object$lev, "in", object$rvar, "\n")
-		cat("Margin:Cost :", object$margin, ":", object$cost, "\n")
-		# cat("AUC         :", paste0(names(auc), " (", round(auc,3), ")", collapse=", "), "\n\n")
-		cat("\n")
-		print(formatdf(as.data.frame(object$dat), 3), row.names = FALSE)
-	} else {
-    return(add_class(object$dat, "confusion"))
-	}
+	cat("Confusion matrix\n")
+	cat("Data       :", object$dataset, "\n")
+	if (object$data_filter %>% gsub("\\s","",.) != "")
+		cat("Filter     :", gsub("\\n","", object$data_filter), "\n")
+	cat("Results for:", object$train, "\n")
+	cat("Predictors :", paste0(object$pred, collapse=", "), "\n")
+	cat("Response   :", object$rvar, "\n")
+  cat("Level      :", object$lev, "in", object$rvar, "\n")
+	cat("Cost:Margin:", object$cost, ":", object$margin, "\n")
+	cat("\n")
+
+	print(formatdf(as.data.frame(object$dat[,1:10]), 3), row.names = FALSE)
+	cat("\n")
+	print(formatdf(as.data.frame(object$dat[,c(1,2, 11:17)]), 3, mark = ","), row.names = FALSE)
 }
 
 #' Plot method for the confusion matrix
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/evalreg.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
 #'
-#' @param x Return value from \code{\link{evalreg}}
+#' @param x Return value from \code{\link{confusion}}
+#' @param vars Measures to plot, i.e., one or more of "TP", "FP", "TN", "FN", "total", "TPR", "TNR", "precision", "accuracy", "kappa", "profit", "index", "ROME", "contact", "AUC"
 #' @param scale_y Free scale in faceted plot of the confusion matrix (TRUE or FALSE)
-#' @param shiny Did the function call originate inside a shiny app
 #' @param ... further arguments passed to or from other methods
 #'
-#' @seealso \code{\link{evalreg}} to generate results
-#' @seealso \code{\link{summary.evalreg}} to summarize results
+#' @seealso \code{\link{confusion}} to generate results
+#' @seealso \code{\link{summary.confusion}} to summarize results
 #'
 #' @export
-plot.confusion <- function(x, scale_y = FALSE, shiny = FALSE, ...) {
+plot.confusion <- function(x, vars = c("kappa", "index", "ROME", "AUC"),
+                           scale_y = TRUE, ...) {
 
 	object <- x; rm(x)
-  if (is.character(object) || is.null(object)) return(invisible())
+ 	if (is.character(object) || is.null(object)) return(invisible())
 
 	dat <- object$dat %>%
-	  mutate_each_(funs(if (is.numeric(.)) . / total else .), vars = c("TN","FN","FP","TP"))
+	  mutate_at(.cols = c("TN","FN","FP","TP"), .funs = funs(if (is.numeric(.)) . / total else .))
 
-	gather_(dat, "Metric", "Value", c("TN","FN","FP","TP","TPR","TNR"), factor_key = TRUE) %>%
+	gather_(dat, "Metric", "Value", vars, factor_key = TRUE) %>%
 		mutate(Predictor = factor(Predictor, levels = unique(Predictor))) %>%
 		{if (scale_y) {
 	    visualize(., xvar = "Predictor", yvar = "Value", type = "bar",
 		    facet_row = "Metric", fill = "Type", axes = "scale_y", custom = TRUE) +
-			  ylab("") + xlab("Predictor")
+			  labs(y = "", x = "Predictor")
 			} else {
 		    visualize(., xvar = "Predictor", yvar = "Value", type = "bar",
 			    facet_row = "Metric", fill = "Type", custom = TRUE) +
-				  ylab("") + xlab("Predictor")
+			  	labs(y = "", x = "Predictor")
 			}
 	  }
 }
 
 #' Plot method for the evalbin function
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
 #'
 #' @param x Return value from \code{\link{evalbin}}
 #' @param plots Plots to return
 #' @param shiny Did the function call originate inside a shiny app
+#' @param custom Logical (TRUE, FALSE) to indicate if ggplot object (or list of ggplot objects) should be returned. This opion can be used to customize plots (e.g., add a title, change x and y labels, etc.). See examples and \url{http://docs.ggplot2.org/} for options.
 #' @param ... further arguments passed to or from other methods
 #'
 #' @seealso \code{\link{evalbin}} to generate results
@@ -349,7 +408,7 @@ plot.confusion <- function(x, scale_y = FALSE, shiny = FALSE, ...) {
 #' evalbin("titanic", c("age","fare"), "survived") %>% summary
 #'
 #' @export
-plot.evalbin <- function(x, plots = c("lift","gains"), shiny = FALSE, ...) {
+plot.evalbin <- function(x, plots = c("lift","gains"), shiny = FALSE, custom = FALSE, ...) {
 
 	## to avoid 'global not defined' warnings
 	pred <- cum_prop <- cum_gains <- obs <- profit <- NULL
@@ -364,46 +423,43 @@ plot.evalbin <- function(x, plots = c("lift","gains"), shiny = FALSE, ...) {
 			visualize(object$dat, xvar = "cum_prop", yvar = "cum_lift", type = "line", color = "pred", custom = TRUE) +
 			geom_point() +
 			geom_segment(aes(x = 0, y = 1, xend = 1, yend = 1), size = .1, color = "black") +
-			ylab("Cumulative lift") +
-			xlab("Proportion of customers")
+			labs(y = "Cumulative lift", x = "Proportion of customers")
 	}
 
 	if ("gains" %in% plots) {
-		init <- object$dat[1,] %>% {.[1,] <- 0; .}
 		dat <-
 	    object$dat %>%
 		  select(pred, cum_prop, cum_gains) %>%
 		  group_by(pred) %>%
 		  mutate(obs = 1:n())
+
 		init <- dat %>% filter(obs == 1)
-		init$cum_prop <- init$cum_gains <- init$obs <- 0
+		init[,c("cum_prop", "cum_gains", "obs")] <- 0 
 		dat <- bind_rows(init, dat) %>% arrange(pred, obs)
 
 		plot_list[["gains"]] <-
 		  visualize(dat, xvar = "cum_prop", yvar = "cum_gains", type = "line", color = "pred", custom = TRUE) +
 			geom_point() +
 			geom_segment(aes(x = 0, y = 0, xend = 1, yend = 1), size = .1, color = "black") +
-			ylab("Cumulative gains") +
-			xlab("Proportion of customers")
+			labs(y = "Cumulative gains", x = "Proportion of customers")
 	}
 
 	if ("profit" %in% plots) {
-		init <- object$dat[1,] %>% {.[1,] <- 0; .}
 		dat <-
 	    object$dat %>%
 		  select(pred, cum_prop, profit) %>%
 		  group_by(pred) %>%
 		  mutate(obs = 1:n())
+
 		init <- dat %>% filter(obs == 1)
-		init$profit <- init$cum_prop <- init$obs <- 0
+		init[,c("profit", "cum_prop", "obs")] <- 0 
 		dat <- bind_rows(init, dat) %>% arrange(pred, obs)
 
 		plot_list[["profit"]] <-
 			visualize(dat, xvar = "cum_prop", yvar = "profit", type = "line", color = "pred", custom = TRUE) +
 			geom_point() +
 			geom_segment(aes(x = 0, y = 0, xend = 1, yend = 0), size = .1, color = "black") +
-			ylab("Profit index") +
-			xlab("Proportion of customers")
+			labs(y = "Profit", x = "Proportion of customers")
 	}
 
 	if ("rome" %in% plots) {
@@ -411,8 +467,7 @@ plot.evalbin <- function(x, plots = c("lift","gains"), shiny = FALSE, ...) {
 			visualize(object$dat, xvar = "cum_prop", yvar = "ROME", type = "line", color = "pred", custom = TRUE) +
 			geom_point() +
 			geom_segment(aes(x = 0, y = 0, xend = 1, yend = 0), size = .1, color = "black") +
-			ylab("Return on Marketing Expenditures (ROME)") +
-			xlab("Proportion of customers")
+			labs(y = "Return on Marketing Expenditures (ROME)", x = "Proportion of customers")
 	}
 
 	for (i in names(plot_list)) {
@@ -422,18 +477,16 @@ plot.evalbin <- function(x, plots = c("lift","gains"), shiny = FALSE, ...) {
 			plot_list[[i]] <- plot_list[[i]] + labs(colour = "Predictor")
 	}
 
-	dots <- list(...)
-  if ("custom" %in% names(dots) && dots$custom == TRUE) {
-    if (length(plot_list) == 1) return(plot_list[[plots]]) else return(plot_list)
-  }
+  if (custom)
+    if (length(plot_list) == 1) return(plot_list[[1]]) else return(plot_list)
 
-	sshhr( do.call(gridExtra::arrangeGrob, c(plot_list, list(ncol = 1))) ) %>%
-	 	{ if (shiny) . else print(.) }
+	sshhr(gridExtra::grid.arrange(grobs = plot_list, ncol = 1)) %>%
+	 	{if (shiny) . else print(.)}
 }
 
 #' Area Under the Curve (AUC)
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/evalbin.html} for an example in Radiant
 #'
 #' @param pred Prediction or predictor
 #' @param rvar Response variable
@@ -446,17 +499,18 @@ plot.evalbin <- function(x, plots = c("lift","gains"), shiny = FALSE, ...) {
 #' @seealso \code{\link{plot.evalbin}} to plot results
 #'
 #' @examples
-#' auc(mtcars$mpg, mtcars$vs, 1)
+#' auc(runif(nrow(mtcars)), mtcars$vs, 1)
 #'
 #' @export
 auc <- function(pred, rvar, lev) {
  	## based on examples in colAUC at ...
  	## https://cran.r-project.org/web/packages/caTools/caTools.pdf
+ 	if (missing(lev) && is.factor(rvar)) lev <- levels(rvar)[1]
  	stopifnot(length(lev) == 1, lev %in% rvar)
  	x1 <- pred[rvar == lev]
   x2 <- pred[rvar != lev]
   ## need as.numeric to avoid integer-overflows
  	denom <- as.numeric(length(x1)) * length(x2)
 	wt <- wilcox.test(x1, x2, exact = FALSE)$statistic / denom
-	ifelse (wt < .5, 1 - wt, wt)
+	ifelse (wt < .5, 1 - wt, wt)[["W"]]
 }
