@@ -1,12 +1,12 @@
 #' Linear regression using OLS
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param dataset Dataset name (string). This can be a dataframe in the global environment or an element in an r_data list from Radiant
 #' @param rvar The response variable in the regression
 #' @param evar Explanatory variables in the regression
 #' @param int Interaction terms to include in the model
-#' @param check "standardize" to see standardized coefficient estimates. "stepwise" to apply step-wise selection of variables in estimation
+#' @param check Use "standardize" to see standardized coefficient estimates. Use "stepwise-backward" (or "stepwise-forward", or "stepwise-both") to apply step-wise selection of variables in estimation
 #' @param data_filter Expression entered in, e.g., Data > View to filter the dataset in Radiant. The expression should be a string (e.g., "price > 10000")
 #'
 #' @return A list of all variables variables used in the regress function as an object of class regress
@@ -30,9 +30,9 @@ regress <- function(dataset, rvar, evar,
            add_class("regress"))
 
   dat <- getdata(dataset, c(rvar, evar), filt = data_filter)
-  if (!is_string(dataset)) dataset <- "-----"
+  if (!is_string(dataset)) dataset <- deparse(substitute(dataset)) %>% set_attr("df", TRUE)
 
-  if (any(summarise_each(dat, funs(does_vary)) == FALSE))
+  if (any(summarise_all(dat, funs(does_vary)) == FALSE))
     return("One or more selected variables show no variation. Please select other variables." %>%
            add_class("regress"))
 
@@ -40,6 +40,9 @@ regress <- function(dataset, rvar, evar,
   var_check(evar, colnames(dat)[-1], int) %>%
     { vars <<- .$vars; evar <<- .$ev; int <<- .$intv }
 
+  ## add minmax attributes to data
+  mmx <- minmax(dat)
+
   ## scale data
   isNum <- sapply(dat, is.numeric)
   if (sum(isNum) > 0) {
@@ -50,16 +53,30 @@ regress <- function(dataset, rvar, evar,
     }
   }
 
-  form <- paste(rvar, "~", paste(vars, collapse = " + ")) %>% as.formula
-
-  if ("stepwise" %in% check) {
+  form_upper <- paste(rvar, "~", paste(vars, collapse = " + ")) %>% as.formula
+  form_lower <- paste(rvar, "~ 1") %>% as.formula
+  if ("stepwise" %in% check) check <- sub("stepwise", "stepwise-backward", check)
+  if ("stepwise-backward" %in% check) {
     ## use k = 2 for AIC, use k = log(nrow(dat)) for BIC
-    # model <- lm(paste(rvar, "~ 1") %>% as.formula, data = dat) %>%
-    ## using backward stepwise selection
-    model <- lm(form, data = dat) %>%
-      step(k = 2, scope = list(upper = form), direction = "backward")
+    model <- lm(form_upper, data = dat) %>%
+      step(k = 2, scope = list(lower = form_lower), direction = "backward")
+
+    ## adding full data even if all variables are not significant
+    model$model <- dat
+  } else if ("stepwise-forward" %in% check) {
+    model <- lm(form_lower, data = dat) %>%
+      step(k = 2, scope = list(upper = form_upper), direction = "forward")
+
+    ## adding full data even if all variables are not significant
+    model$model <- dat
+  } else if ("stepwise-both" %in% check) {
+    model <- lm(form_lower, data = dat) %>%
+      step(k = 2, scope = list(lower = form_lower, upper = form_upper), direction = "both")
+
+    ## adding full data even if all variables are not significant
+    model$model <- dat
   } else {
-    model <- lm(form, data = dat)
+    model <- lm(form_upper, data = dat)
   }
 
   ## needed for prediction if standardization or centering is used
@@ -68,7 +85,11 @@ regress <- function(dataset, rvar, evar,
     attr(model$model, "sds") <- attr(dat, "sds")
   }
 
+  attr(model$model, "min") <- mmx[["min"]]
+  attr(model$model, "max") <- mmx[["max"]]
+
   coeff <- tidy(model)
+
   coeff$` ` <- sig_stars(coeff$p.value) %>% format(justify = "left")
   colnames(coeff) <- c("  ","coefficient","std.error","t.value","p.value"," ")
   isFct <- sapply(select(dat,-1), function(x) is.factor(x) || is.logical(x))
@@ -87,7 +108,7 @@ regress <- function(dataset, rvar, evar,
 
 #' Summary method for the regress function
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param object Return value from \code{\link{regress}}
 #' @param sum_check Optional output. "rsme" to show the root mean squared error and the standard deviation of the residuals. "sumsquares" to show the sum of squares table. "vif" to show multicollinearity diagnostics. "confint" to show coefficient confidence interval estimates.
@@ -120,10 +141,13 @@ summary.regress <- function(object,
   if (is.character(object)) return(object)
   if (class(object$model)[1] != 'lm') return(object)
 
-  if ("stepwise" %in% object$check) {
-    cat("-----------------------------------------------\n")
-    cat("Backward stepwise selection of variables\n")
-    cat("-----------------------------------------------\n")
+  if (any(grepl("stepwise", object$check))) {
+    step_type <- if ("stepwise-backward" %in% object$check) "Backward"
+      else if ("stepwise-forward" %in% object$check) "Forward"
+      else "Forward and Backward"
+    cat("----------------------------------------------------\n")
+    cat(step_type, "stepwise selection of variables\n")
+    cat("----------------------------------------------------\n")
   }
 
   cat("Linear regression (OLS)\n")
@@ -142,7 +166,6 @@ summary.regress <- function(object,
   }
 
   coeff <- object$coeff
-  # object$coeff$p.value <- "NaN"
   cat("\n")
   if (all(object$coeff$p.value == "NaN")) {
     coeff[,2] %<>% {sprintf(paste0("%.",dec,"f"),.)}
@@ -162,6 +185,9 @@ summary.regress <- function(object,
   ## adjusting df for included intercept term
   df_int <- if (attr(object$model$terms, "intercept")) 1L else 0L
 
+  ## if stepwise returns only an intercept
+  if (nrow(coeff) == 1) return("\nModel contains only an intercept. No additional output shown")
+
   reg_fit <- glance(object$model) %>% round(dec)
   if (reg_fit['p.value'] < .001) reg_fit['p.value'] <- "< .001"
   cat("\nSignif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1\n\n")
@@ -205,14 +231,14 @@ summary.regress <- function(object,
       if (length(object$evar) > 1) {
         cat("Variance Inflation Factors\n")
         car::vif(object$model) %>%
-          { if (!dim(.) %>% is.null) .[,"GVIF"] else . } %>% ## needed when factors are included
+          {if (is.null(dim(.))) . else .[,"GVIF^(1/(2*Df))"]} %>% ## needed when factors are included
           data.frame("VIF" = ., "Rsq" = 1 - 1/.) %>%
           round(dec) %>%
           .[order(.$VIF, decreasing=T),] %>%
           { if (nrow(.) < 8) t(.) else . } %>%
           print
       } else {
-        cat("Insufficient explanatory variables to calculate\nmulticollinearity diagnostics (VIF)")
+        cat("Insufficient number of explanatory variables to calculate\nmulticollinearity diagnostics (VIF)\n")
       }
     }
     cat("\n")
@@ -228,7 +254,7 @@ summary.regress <- function(object,
         as.data.frame %>%
         set_colnames(c("Low","High")) %>%
         { .$`+/-` <- (.$High - .$Low)/2; . } %>%
-        mutate_each(funs(sprintf(paste0("%.",dec,"f"),.))) %>%
+        mutate_all(funs(sprintf(paste0("%.",dec,"f"),.))) %>%
         cbind(coeff[[2]],.) %>%
         set_rownames(object$coeff$`  `) %>%
         set_colnames(c("coefficient", ci_perc[1], ci_perc[2], "+/-")) %T>%
@@ -238,7 +264,7 @@ summary.regress <- function(object,
   }
 
   if (!is.null(test_var) && test_var[1] != "") {
-    if ("stepwise" %in% object$check) {
+    if (any(grepl("stepwise", object$check))) {
       cat("Model comparisons are not conducted when Stepwise has been selected.\n")
     } else {
       sub_form <- ". ~ 1"
@@ -271,10 +297,10 @@ summary.regress <- function(object,
 
 #' Plot method for the regress function
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param x Return value from \code{\link{regress}}
-#' @param plots Regression plots to produce for the specified regression model. Enter "" to avoid showing any plots (default). "hist" to show histograms of all variables in the model. "correlations" for a visual representation of the correlation matrix selected variables. "scatter" to show scatter plots (or box plots for factors) for the response variable with each explanatory variable. "dashboard" for a series of six plots that can be used to evaluate model fit visually. "resid_pred" to plot the explanatory variables against the model residuals. "coef" for a coefficient plot with adjustable confidence intervals. "leverage" to show leverage plots for each explanatory variable
+#' @param plots Regression plots to produce for the specified regression model. Enter "" to avoid showing any plots (default). "dist" to shows histograms (or frequency bar plots) of all variables in the model. "correlations" for a visual representation of the correlation matrix selected variables. "scatter" to show scatter plots (or box plots for factors) for the response variable with each explanatory variable. "dashboard" for a series of six plots that can be used to evaluate model fit visually. "resid_pred" to plot the explanatory variables against the model residuals. "coef" for a coefficient plot with adjustable confidence intervals. "leverage" to show leverage plots for each explanatory variable
 #' @param lines Optional lines to include in the select plot. "line" to include a line through a scatter plot. "loess" to include a polynomial regression fit line. To include both use c("line","loess")
 #' @param conf_lev Confidence level used to estimate confidence intervals (.95 is the default)
 #' @param intercept Include the intercept in the coefficient plot (TRUE, FALSE). FALSE is the default
@@ -284,13 +310,13 @@ summary.regress <- function(object,
 #'
 #' @examples
 #' result <- regress("diamonds", "price", c("carat","clarity"))
-#' plot(result, plots = "dashboard", lines = c("line","loess"))
 #' plot(result, plots = "coef", conf_lev = .99, intercept = TRUE)
-#' plot(result, plots = "hist")
+#' plot(result, plots = "dist")
+#' \dontrun{
 #' plot(result, plots = "scatter", lines = c("line","loess"))
-#' plot(result, plots = "correlations")
 #' plot(result, plots = "resid_pred", lines = "line")
-#'
+#' plot(result, plots = "dashboard", lines = c("line","loess"))
+#' }
 #' @seealso \code{\link{regress}} to generate the results
 #' @seealso \code{\link{summary.regress}} to summarize results
 #' @seealso \code{\link{predict.regress}} to generate predictions
@@ -309,7 +335,6 @@ plot.regress <- function(x, plots = "",
   if (class(object$model)[1] != 'lm') return(object)
 
   ## checking object size
-  # object_size(object$model, model)
   model <- ggplot2::fortify(object$model)
 
   rvar <- object$rvar
@@ -320,47 +345,48 @@ plot.regress <- function(x, plots = "",
   nlines <- sub("jitter","",lines)
 
   plot_list <- list()
-  if ("hist" %in% plots)
-    for (i in vars) {
-      plot_list[[paste0("hist",i)]] <-
-        visualize(select_(model, .dots = i), xvar = i, bins = 10, custom = TRUE)
-    }
-
   if ("dashboard" %in% plots) {
 
-    plot_list[[1]] <-
+    plot_list[["dash1"]] <-
       visualize(model, xvar = ".fitted", yvar = rvar, type = "scatter", custom = TRUE) +
       labs(list(title = "Actual vs Fitted values", x = "Fitted", y = "Actual"))
 
-    plot_list[[2]] <-
+    plot_list[["dash2"]] <-
       visualize(model, xvar = ".fitted", yvar = ".resid", type = "scatter", custom = TRUE) +
       labs(list(title = "Residuals vs Fitted", x = "Fitted values", y = "Residuals"))
 
-    plot_list[[3]] <- ggplot(model, aes(y=.resid, x=seq_along(.resid))) + geom_line() +
+    plot_list[["dash3"]] <- ggplot(model, aes(y=.resid, x=seq_along(.resid))) + geom_line() +
       labs(list(title = "Residuals vs Row order", x = "Row order", y = "Residuals"))
 
-    plot_list[[4]] <- ggplot(model, aes_string(sample=".stdresid")) + stat_qq(alpha = .5) +
+    plot_list[["dash4"]] <- ggplot(model, aes_string(sample=".stdresid")) + stat_qq(alpha = .5) +
       labs(list(title = "Normal Q-Q", x = "Theoretical quantiles", y = "Standardized residuals"))
 
-    plot_list[[5]] <-
+    plot_list[["dash5"]] <-
       visualize(model, xvar = ".resid", custom = TRUE) +
       labs(list(title = "Histogram of residuals", x = "Residuals"))
 
-    plot_list[[6]] <- ggplot(model, aes_string(x=".resid")) + geom_density(alpha=.3, fill = "green") +
+    plot_list[["dash6"]] <- ggplot(model, aes_string(x=".resid")) + geom_density(alpha=.3, fill = "green") +
       stat_function(fun = dnorm, args = list(mean = mean(model[,'.resid']), sd = sd(model[,'.resid'])), color = "blue") +
       labs(list(title = "Residuals vs Normal density", x = "Residuals", y = "")) + theme(axis.text.y = element_blank())
 
     if ("loess" %in% lines)
-      for (i in 1:3) plot_list[[i]] <- plot_list[[i]] + sshhr( geom_smooth(method = "loess", size = .75, linetype = "dotdash") )
+      for (i in paste0("dash",1:3)) plot_list[[i]] <- plot_list[[i]] + sshhr( geom_smooth(method = "loess", size = .75, linetype = "dotdash") )
 
     if ("line" %in% lines) {
-      for (i in c(1,4))
+      for (i in paste0("dash",c(1,4)))
         plot_list[[i]] <- plot_list[[i]] + geom_abline(linetype = 'dotdash')
-      for (i in 2:3)
+      for (i in paste0("dash",2:3))
         plot_list[[i]] <- plot_list[[i]] + sshhr( geom_smooth(method = "lm", se = FALSE, size = .75, linetype = "dotdash", colour = 'black') )
     }
   }
 
+  if ("dist" %in% plots) {
+    for (i in vars) {
+      plot_list[[paste0("dist",i)]] <-
+        visualize(select_(model, .dots = i), xvar = i, bins = 10, custom = TRUE)
+    }
+  }
+
   if ("scatter" %in% plots) {
     for (i in evar) {
       if ("factor" %in% class(model[,i])) {
@@ -376,11 +402,11 @@ plot.regress <- function(x, plots = "",
   if ("resid_pred" %in% plots) {
     for (i in evar) {
       if ("factor" %in% class(model[,i])) {
-        plot_list[[i]] <-
+        plot_list[[paste0("resid_",i)]] <-
           visualize(select_(model, .dots = c(i,".resid")), xvar = i, yvar = ".resid", type = "scatter", check = flines, alpha = .2, custom = TRUE) +
           ylab("residuals")
       } else {
-        plot_list[[i]] <-
+        plot_list[[paste0("resid_",i)]] <-
           visualize(select_(model, .dots = c(i,".resid")), xvar = i, yvar = ".resid", type = "scatter", check = nlines, custom = TRUE) +
           ylab("residuals")
       }
@@ -389,10 +415,11 @@ plot.regress <- function(x, plots = "",
 
   if ("coef" %in% plots) {
 
+    if (nrow(object$coeff) == 1 && !intercept) return("** Model contains only an intercept **")
+
     yl <- if ("standardize" %in% object$check) "Coefficient (standardized)" else "Coefficient"
 
-    p <-
-      confint(object$model, level = conf_lev) %>%
+    plot_list[["coeff"]] <- confint(object$model, level = conf_lev) %>%
       data.frame %>%
       na.omit %>%
       set_colnames(c("Low","High")) %>%
@@ -404,11 +431,9 @@ plot.regress <- function(x, plots = "",
           geom_pointrange(aes_string(x = "variable", y = "coefficient",
                           ymin = "Low", ymax = "High")) +
           geom_hline(yintercept = 0, linetype = 'dotdash', color = "blue") +
-          xlab("") +
-          ylab(yl) +
+          labs(x = "", y = yl) +
           scale_x_discrete(limits = {if (intercept) rev(object$coeff$`  `) else rev(object$coeff$`  `[-1])}) +
           coord_flip() + theme(axis.text.y = element_text(hjust = 0))
-    return(p)
   }
 
   if ("correlations" %in% plots)
@@ -422,18 +447,18 @@ plot.regress <- function(x, plots = "",
   #          layout = c(ceiling(length(evar)/2),2)))
   # }
 
-  if (custom)
-    if (length(plot_list) == 1) return(plot_list[[1]]) else return(plot_list)
+  if (length("plot_list") > 0) {
+    if (custom)
+      if (length(plot_list) == 1) return(plot_list[[1]]) else return(plot_list)
 
-  if (exists("plot_list")) {
-    sshhr( do.call(gridExtra::arrangeGrob, c(plot_list, list(ncol = 2))) ) %>%
-        {if (shiny) . else print(.)}
+    sshhr(gridExtra::grid.arrange(grobs = plot_list, ncol = 2)) %>%
+       {if (shiny) . else print(.)}
   }
 }
 
 #' Predict method for the regress function
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param object Return value from \code{\link{regress}}
 #' @param pred_data Name of the dataset to use for prediction
@@ -465,6 +490,7 @@ predict.regress <- function(object,
                             dec = 3,
                             ...) {
 
+ if (is.character(object)) return(object)
  if ("center" %in% object$check || "standardize" %in% object$check) se <- FALSE
 
  pfun <- function(model, pred, se, conf_lev) {
@@ -489,12 +515,12 @@ predict.regress <- function(object,
     pred_val
   }
 
-  predict.model(object, pfun, "regress.predict", pred_data, pred_cmd, conf_lev, se, dec)
+  predict_model(object, pfun, "regress.predict", pred_data, pred_cmd, conf_lev, se, dec)
 }
 
 #' Predict method for model functions
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param object Return value from \code{\link{regress}}
 #' @param pfun Function to use for prediction
@@ -509,7 +535,7 @@ predict.regress <- function(object,
 #' @importFrom radiant.data set_attr
 #'
 #' @export
-predict.model <- function(object, pfun, mclass,
+predict_model <- function(object, pfun, mclass,
                           pred_data = "",
                           pred_cmd = "",
                           conf_lev = 0.95,
@@ -522,22 +548,34 @@ predict.model <- function(object, pfun, mclass,
   if (is_empty(pred_data) && is_empty(pred_cmd))
     return("Please select data and/or specify a command to generate predictions.\nFor example, carat = seq(.5, 1.5, .1) would produce predictions for values\n of carat starting at .5, increasing to 1.5 in increments of .1. Make sure\nto press return after you finish entering the command.\n\nAlternatively, specify a dataset to generate predictions. You could create\nthis in a spread sheet and use the paste feature in Data > Manage to bring\nit into Radiant")
 
-  # if ("standardize" %in% object$check)
-  #   return("Standardized coefficients will not be used for prediction.\nPlease uncheck the 'standardize' box and try again")
-
-  # if ("center" %in% object$check)
-  #   return("Centered coefficients will not be used for prediction.\nPlease uncheck the 'center' box and try again")
-
   pred_type <- "cmd"
   vars <- object$evar
   if (is_empty(pred_data) && !is_empty(pred_cmd)) {
+
+    dat <- object$model$model
+    if ("center" %in% object$check) {
+      ms <- attr(object$model$model, "ms")
+      if (!is.null(ms))
+        dat <- mutate_at(dat, .cols = names(ms), .funs = funs(. + ms$.))
+
+    } else if ("standardize" %in% object$check) {
+      ms <- attr(object$model$model, "ms")
+      sds <- attr(object$model$model,"sds")
+      if (!is.null(ms) && !is.null(sds))
+        dat <- mutate_at(dat, .cols = names(ms), .funs = funs(. * 2 * sds$. + ms$.))
+    }
+
     pred_cmd %<>% gsub("\"","\'",.) %>% gsub(";\\s*$","",.) %>% gsub(";",",",.)
-    pred <- try(eval(parse(text = paste0("with(object$model$model, expand.grid(", pred_cmd ,"))"))), silent = TRUE)
+    pred <- try(eval(parse(text = paste0("with(dat, expand.grid(", pred_cmd ,"))"))), silent = TRUE)
     if (is(pred, 'try-error'))
       return(paste0("The command entered did not generate valid data for prediction. The\nerror message was:\n\n", attr(pred,"condition")$message, "\n\nPlease try again. Examples are shown in the help file."))
 
     # adding information to the prediction data.frame
-    dat_classes <- attr(object$model$term, "dataClasses")[-1]
+    dat <- select_(dat, .dots = vars)
+    if (!is.null(object$model$term))
+      dat_classes <- attr(object$model$term, "dataClasses")[-1]
+    else
+      dat_classes <- getclass(dat)
 
     ## weights mess-up data manipulation below so remove from
     wid <- which(names(dat_classes) %in% "(weights)")
@@ -545,20 +583,19 @@ predict.model <- function(object, pfun, mclass,
 
     isFct <- dat_classes == "factor"
     isLog <- dat_classes == "logical"
-    isNum <- dat_classes == "numeric"
-    dat <- select_(object$model$model, .dots = vars)
+    isNum <- dat_classes == "numeric" | dat_classes == "integer"
 
     # based on http://stackoverflow.com/questions/19982938/how-to-find-the-most-frequent-values-across-several-columns-containing-factors
-    max_freq <- function(x) names(which.max(table(x)))
+    max_freq <- function(x) names(which.max(table(x))) %>% as.factor
     max_lfreq <- function(x) ifelse(mean(x) > .5, TRUE, FALSE)
 
     plug_data <- data.frame(init___ = 1)
     if (sum(isNum) > 0)
-      plug_data %<>% bind_cols(., summarise_each_(dat, funs(mean), vars[isNum]))
+      plug_data %<>% bind_cols(., summarise_at(dat, .cols = vars[isNum], .funs = funs(mean)))
     if (sum(isFct) > 0)
-      plug_data %<>% bind_cols(., summarise_each_(dat, funs(max_freq), vars[isFct]))
+      plug_data %<>% bind_cols(., summarise_at(dat, .cols = vars[isFct], .funs = funs(max_freq)))
     if (sum(isLog) > 0)
-      plug_data %<>% bind_cols(., summarise_each_(dat, funs(max_lfreq), vars[isLog]))
+      plug_data %<>% bind_cols(., summarise_at(dat, .cols = vars[isLog], .funs = funs(max_lfreq)))
 
     rm(dat)
 
@@ -624,7 +661,7 @@ predict.model <- function(object, pfun, mclass,
     if ("center" %in% object$check) {
       ms <- attr(object$model$model, "ms")[[object$rvar]]
       if (!is.null(ms))
-        pred_val[["Prediction"]] <- pred_val[["Prediction"]]  + ms
+        pred_val[["Prediction"]] <- pred_val[["Prediction"]] + ms
     } else if ("standardize" %in% object$check) {
       ms <- attr(object$model$model, "ms")[[object$rvar]]
       sds <- attr(object$model$model,"sds")[[object$rvar]]
@@ -635,7 +672,7 @@ predict.model <- function(object, pfun, mclass,
     pred <- data.frame(pred, pred_val, check.names = FALSE)
     vars <- colnames(pred)
 
-    if ("stepwise" %in% object$check) {
+    if (any(grepl("stepwise", object$check))) {
       ## show only the selected variables when printing predictions
       object$evar <- attr(terms(object$model), "variables") %>% as.character %>% .[-c(1,2)]
       vars <- c(object$evar, colnames(pred_val))
@@ -645,7 +682,9 @@ predict.model <- function(object, pfun, mclass,
     pred <- set_attr(pred, "dataset", object$dataset) %>%
       set_attr("data_filter", object$data_filter) %>%
       set_attr("rvar", object$rvar) %>%
+      set_attr("lev", object$lev) %>%
       set_attr("evar", object$evar) %>%
+      set_attr("wtsname", object$wtsname) %>%
       set_attr("vars", vars) %>%
       set_attr("dec", dec) %>%
       set_attr("pred_type", pred_type) %>%
@@ -664,27 +703,32 @@ predict.model <- function(object, pfun, mclass,
 #' @param ... further arguments passed to or from other methods
 #' @param n Number of lines of prediction results to print. Use -1 to print all lines
 #' @param header Header line
-#' @param lev The level in the response variable defined as _success_ for classification models
 #'
 #' @export
-print.model.predict <- function(x, ..., n = 10, header = "", lev = "") {
+print_predict_model <- function(x, ..., n = 10, header = "") {
 
+  class(x) <- "data.frame"
   data_filter <- attr(x, "data_filter")
   vars <- attr(x, "vars")
   pred_type <- attr(x, "pred_type")
   pred_data <- attr(x, "pred_data")
-  pred_cmd <- gsub("([=+-/*])", " \\1 ", attr(x, "pred_cmd")) %>% gsub("([;])", "\\1 ", .)
+
+  pred_cmd <- gsub("([\\=\\+\\*-])", " \\1 ", attr(x, "pred_cmd")) %>%
+    gsub("([;,])", "\\1 ", .) %>%
+    gsub("\\s+=\\s+=\\s+", " == ", .)
 
   cat(header)
   cat("\nData                 :", attr(x, "dataset"), "\n")
   if (data_filter %>% gsub("\\s","",.) != "")
     cat("Filter               :", gsub("\\n","", data_filter), "\n")
   cat("Response variable    :", attr(x, "rvar"), "\n")
-  if (!is_empty(lev))
-    cat("Level                :", lev, "in", attr(x, "rvar"), "\n")
+  if (!is_empty(attr(x, "lev")))
+    cat("Level(s)             :", paste0(attr(x, "lev"), collapse = ", "), "in", attr(x, "rvar"), "\n")
   cat("Explanatory variables:", paste0(attr(x, "evar"), collapse=", "), "\n")
+  if (!is_empty(attr(x, "wtsname")))
+    cat("Weights used         :", attr(x, "wtsname"), "\n")
 
-  if (!is.character(pred_data)) pred_data <- "-----"
+  if (!is.character(pred_data)) pred_data <- deparse(substitute(pred_data)) %>% set_attr("df", TRUE)
   if (pred_type == "cmd") {
     cat("Prediction command   :", pred_cmd, "\n")
   } else if (pred_type == "datacmd") {
@@ -696,12 +740,12 @@ print.model.predict <- function(x, ..., n = 10, header = "", lev = "") {
 
   if (n == -1) {
     cat("\n")
-    formatdf(x[,vars], attr(x, "dec")) %>% print(row.names = FALSE)
+    formatdf(x[,vars, drop = FALSE], attr(x, "dec")) %>% print(row.names = FALSE)
   } else {
     if (nrow(x) > n)
-      cat("Rows shown           :", n, "\n")
+      cat("Rows shown           :", n, "of", formatnr(nrow(x), dec = 0), "\n")
     cat("\n")
-    head(x[,vars], n) %>% formatdf(attr(x, "dec")) %>% print(row.names = FALSE)
+    head(x[,vars, drop = FALSE], n) %>% formatdf(attr(x, "dec")) %>% print(row.names = FALSE)
   }
 }
 
@@ -713,7 +757,7 @@ print.model.predict <- function(x, ..., n = 10, header = "", lev = "") {
 #'
 #' @export
 print.regress.predict <- function(x, ..., n = 10)
-  print.model.predict(x, ..., n = n, header = "Linear regression (OLS)")
+  print_predict_model(x, ..., n = n, header = "Linear regression (OLS)")
 
 #' Plot method for model.predict functions
 #'
@@ -747,6 +791,9 @@ plot.model.predict <- function(x, xvar = "",
   ## should work with req in regress_ui but doesn't
   if (is_empty(xvar)) return(invisible())
 
+  if (facet_col != "." && facet_row == facet_col)
+    return("The same variable cannot be used for both Facet row and Facet column")
+
   object <- x; rm(x)
   if (is.character(object)) return(object)
 
@@ -769,24 +816,37 @@ plot.model.predict <- function(x, xvar = "",
     byvar <- if (is.null(byvar)) facet_col else unique(c(byvar, facet_col))
 
   tbv <- if (is.null(byvar)) xvar else c(xvar, byvar)
-  tmp <- object %>% group_by_(.dots = tbv) %>% select_(.dots = c(tbv, pvars)) %>% summarise_each(funs(mean))
-  if (color == 'none') {
-    p <- ggplot(tmp, aes_string(x=xvar, y="Prediction")) + geom_line(aes(group = 1))
-  } else {
-    p <- ggplot(tmp, aes_string(x=xvar, y="Prediction", color = color, group = color)) + geom_line()
-  }
 
-  if (facet_row != "." || facet_col != ".") {
-    facets <- ifelse (facet_row == ".", paste("~", facet_col), paste(facet_row, '~', facet_col))
-    facet_fun <- ifelse (facet_row == ".", facet_wrap, facet_grid)
-    p <- p + facet_fun(as.formula(facets))
+  if ( any(!tbv %in% colnames(object)))
+    return("Some specified plotting variables are not in the model.\nPress the Estimate button to update results.")
+
+  tmp <- object %>% group_by_(.dots = tbv) %>% select_(.dots = c(tbv, pvars)) %>% 
+    summarise_all(funs(mean))
+  if (color == 'none') {
+    p <- ggplot(tmp, aes_string(x = xvar, y = "Prediction"))
+  } else {
+    p <- ggplot(tmp, aes_string(x = xvar, y = "Prediction", color = color, group = color))
   }
 
   if (length(pvars) == 3) {
     if (is.factor(tmp[[xvar]]) || length(unique(tmp[[xvar]])) < 11)
       p <- p + geom_pointrange(aes_string(ymin = "ymin", ymax = "ymax"), size=.3)
     else
-      p <- p + geom_smooth(aes_string(ymin = "ymin", ymax = "ymax"), stat="identity")
+      p <- p + geom_ribbon(aes_string(ymin = "ymin", ymax = "ymax"), fill = "grey70", color = NA, alpha = .5)
+  }
+
+  ## needed now that geom_smooth no longer accepts ymin and ymax as arguments
+  ## can't see line properly using geom_ribbon
+  if (color == 'none') {
+    p <- p + geom_line(aes(group = 1))
+  } else {
+    p <- p + geom_line()
+  }
+
+  if (facet_row != "." || facet_col != ".") {
+    facets <- ifelse (facet_row == ".", paste("~", facet_col), paste(facet_row, '~', facet_col))
+    facet_fun <- ifelse (facet_row == ".", facet_wrap, facet_grid)
+    p <- p + facet_fun(as.formula(facets))
   }
 
   sshhr(p)
@@ -813,7 +873,7 @@ store_reg <- function(object, data = object$dataset,
 
 #' Store predicted values generated in model functions
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param object Return value from model function
 #' @param ... Additional arguments
@@ -821,7 +881,7 @@ store_reg <- function(object, data = object$dataset,
 #' @param name Variable name(s) assigned to predicted values
 #'
 #' @examples
-#' regress(diamonds, rvar = "price", evar = c("carat","cut")) %>%
+#' regress(diamonds, rvar = "price", evar = c("carat","cut"))  %>%
 #'   predict(diamonds) %>%
 #'   store(name = "pred, pred_low, pred_high") %>% head
 #'
@@ -843,7 +903,7 @@ store.model.predict <- function(object, ..., data = attr(object,"pred_data"), na
   }
 
   vars <- colnames(object)[1:(ind-1)]
-  indr <- indexr(data, vars, "")
+  indr <- indexr(data, vars, "", cmd = attr(object, "pred_cmd"))
   pred <- as_data_frame(matrix(NA, nrow = indr$nr, ncol = ncol(df)))
   pred[indr$ind, ] <- as.vector(df) ## as.vector removes all attributes from df
 
@@ -852,7 +912,7 @@ store.model.predict <- function(object, ..., data = attr(object,"pred_data"), na
 
 #' Store residuals from a model
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param object Return value from a model function
 #' @param ... Additional arguments
@@ -865,7 +925,7 @@ store.model.predict <- function(object, ..., data = attr(object,"pred_data"), na
 #' @export
 store.model <- function(object, ..., name = "residuals") {
   if (is_empty(name)) name <- "residuals"
-  dat <- {if (object$dataset == "-----") object$model$model else object$dataset}
+  dat <- if (length(attr(object$dataset, "df")) > 0) object$model$model else object$dataset
   indr <- indexr(dat, c(object$rvar, object$evar), object$data_filter)
   res <- rep(NA, indr$nr)
   res[indr$ind] <- object$model$residuals
@@ -874,7 +934,7 @@ store.model <- function(object, ..., name = "residuals") {
 
 #' Check if main effects for all interaction effects are included in the model
 #' If ':' is used to select a range _evar_ is updated
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param ev List of explanatory variables provided to _regress_ or _logistic_
 #' @param cn Column names for all explanatory variables in _dat_
@@ -907,7 +967,7 @@ var_check <- function(ev, cn, intv = "") {
 
 #' Add interaction terms to list of test variables if needed
 #'
-#' @details See \url{http://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
+#' @details See \url{https://radiant-rstats.github.io/docs/model/regress.html} for an example in Radiant
 #'
 #' @param test_var List of variables to use for testing for regress or logistic
 #' @param int Interaction terms specified
