@@ -32,7 +32,7 @@ output$ui_dtree_list <- renderUI({
 })
 
 output$ui_dtree_name <- renderUI({
-  dtree_name <- input$dtree_list
+  dtree_name <- input$dtree_list[1]
   if (length(dtree_name) == 0) dtree_name <- dtree_name()
   if (is_empty(dtree_name)) dtree_name <- "dtree"
   textInput("dtree_name", NULL, dtree_name, width = "100px")
@@ -47,8 +47,10 @@ output$ui_dtree_sense_name <- renderUI({
 
   dte <- dtree_eval()
   if (length(dte) < 2) return(HTML("No variables are available for sensitivity analysis. If the input file does contain a variables section, press the Calculate button to show the list of available variables."))
-  vars <- dte$vars
+  vars <- dte$yl$variables
   if (is_empty(vars)) return(HTML("No variables are available for sensitivity analysis. If the input file does contain a variables section, press the Calculate button to show the list of available variables."))
+  vars <- vars[!is.na(sshhr(sapply(vars, as.numeric)))]
+  if (length(vars) == 0) return(HTML("No variables are available for sensitivity analysis. If the input file does contain a variables section, press the Calculate button to show the list of available variables."))
   vars[names(vars)] <- names(vars)
 
   selectInput("dtree_sense_name", label = "Sensitivity to changes in:",
@@ -138,12 +140,15 @@ output$dtree <- renderUI({
       td(radioButtons(inputId = "dtree_final", label = "Plot decision tree:",
         c("Initial" = FALSE, "Final" = TRUE),
         selected = state_init("dtree_final", FALSE), inline = TRUE)),
+      td(HTML("&nbsp;&nbsp;&nbsp;")),
+      td(radioButtons(inputId = "dtree_orient", label = "Plot direction:",
+        c("Left-right" = "LR", "Top-down" = "TD"), inline = TRUE)),
       td(actionButton("dtree_eval_plot", "Calculate"), style="padding-top:30px;"),
       td(numericInput("dtree_dec", "Decimals", value = state_init("dtree_dec", 2),
          min = 0, max = 10, width = "70px")),
       td(textInput("dtree_symbol", "Symbol", state_init("dtree_symbol", "$"), width = "70px"))
     )),
-    DiagrammeR::DiagrammeROutput("dtree_plot", height = "600px")
+    DiagrammeR::DiagrammeROutput("dtree_plot", width = "100%", height = "100%")
   ),
   tabPanel("Sensitivity",
     sidebarLayout(
@@ -188,7 +193,7 @@ observe({
 
 dtree_name <- function() {
   isolate({
-    dtree_name <- input$dtree_name
+    dtree_name <- gsub("[^ A-z0-9_\\.\\-]", " ", input$dtree_name) %>% gsub("\\s{2,}", " ", .) %>% gsub("(^\\s+)|(\\s+$)","",.)
     if (is_empty(dtree_name)) {
       dtree_name <- stringr::str_match(input$dtree_edit, "\\s*name:\\s*(.*)\\n\\s*type:")[2]
       if (is.na(dtree_name)) {
@@ -204,11 +209,12 @@ dtree_name <- function() {
 
 dtree_eval <- eventReactive(vals_dtree$dtree_run > 1, {
   req(vals_dtree$dtree_run != 1)
+
   ## update settings and get data.tree name
   dtree_name <- dtree_namer()
 
   if (input$dtree_edit != "") {
-    withProgress(message = 'Creating decision tree', value = 1, {
+    withProgress(message = "Creating decision tree", value = 1, {
       dtree(input$dtree_edit, opt = input$dtree_opt)
     })
   }
@@ -233,13 +239,9 @@ output$dtree_plot <- DiagrammeR::renderDiagrammeR({
   if (is_empty(input$dtree_final)) return(invisible())
   dt <- dtree_eval()
   if (is.null(dt)) {
-    return(invisible())
+    invisible()
   } else {
     pinp <- dtree_plot_inputs()
-    # pinp$shiny <- TRUE
-    # DiagrammeR::DiagrammeR(do.call(plot, c(list(x = dt), pinp)))
-
-    pinp$shiny <- TRUE
     do.call(plot, c(list(x = dt), pinp))
   }
 })
@@ -302,6 +304,10 @@ observeEvent(input$dtree_load_yaml, {
   ## loading yaml file from disk
   inFile <- input$dtree_load_yaml
   yaml_file <- paste0(readLines(inFile$datapath), collapse = "\n")
+
+  ## remove characters that may cause problems in shinyAce
+  yaml_file %<>% gsub("[\x80-\xFF]", "", .) %>% gsub("\r","\n",.)
+
   dtree_name <- sub(paste0(".",tools::file_ext(inFile$name)),"",inFile$name)
   r_data[[dtree_name]] <- yaml_file
   r_data[["dtree_list"]] <- c(dtree_name, r_data[["dtree_list"]]) %>% unique
@@ -311,12 +317,12 @@ observeEvent(input$dtree_load_yaml, {
 
 observeEvent(input$dtree_list, {
   isolate({
-    dtree_name <- input$dtree_name
+    dtree_name <- gsub("[^ A-z0-9_\\.\\-]", " ", input$dtree_name) %>% gsub("\\s{2,}", " ", .) %>% gsub("(^\\s+)|(\\s+$)","",.)
     if (is_empty(dtree_name)) dtree_name <- dtree_name()
     r_data[[dtree_name]] <- input$dtree_edit
   })
 
-  shinyAce::updateAceEditor(session, "dtree_edit", value = r_data[[input$dtree_list]])
+  shinyAce::updateAceEditor(session, "dtree_edit", value = r_data[[input$dtree_list[1]]])
 })
 
 observeEvent(input$dtree_report1, {
@@ -339,8 +345,9 @@ observeEvent(input$dtree_edit, {
 
 
 dtree_namer <- reactive({
-  dtree_name <- input$dtree_name
-  if (is_empty(dtree_name)) dtree_name <- input$dtree_list
+  dtree_name <- gsub("[^ A-z0-9_\\.\\-]", " ", input$dtree_name) %>% gsub("\\s{2,}", " ", .) %>% gsub("(^\\s+)|(\\s+$)","",.)
+
+  if (is_empty(dtree_name)) dtree_name <- input$dtree_list[1]
   if (is_empty(dtree_name)) dtree_name <- dtree_name()
 
   r_data[[dtree_name]] <- input$dtree_edit
@@ -351,7 +358,7 @@ dtree_namer <- reactive({
 
 ## remove yaml input
 observeEvent(input$dtree_remove, {
-  dtree_name <- input$dtree_list
+  dtree_name <- input$dtree_list[1]
   r_data[["dtree_list"]] <- setdiff(r_data[["dtree_list"]], dtree_name)
   r_data[[dtree_name]] <- NULL
 })
@@ -362,7 +369,7 @@ observeEvent(input$dtree_remove, {
   inp_out <- list("", "")
   figs <- FALSE
   if (!is_empty(input$dtree_sense) && !is_not(input$dtree_sense_decision)) {
-    inp_out[[2]] <- list(vars = gsub("\n+", "", input$dtree_sense), decs = input$dtree_sense_decision)
+    inp_out[[2]] <- list(vars = gsub("\n+", "", input$dtree_sense), decs = input$dtree_sense_decision, custom = FALSE)
     outputs <- c(outputs, "sensitivity")
     figs <- TRUE
   }
@@ -377,8 +384,10 @@ observeEvent(input$dtree_remove, {
      else paste0(sub("list(", "render(plot(result, ", ., fixed = TRUE),")")} %>%
      gsub("[\"\']TRUE[\'\"]", "TRUE", .)
 
-  # update_report(inp_main = list(yl = dtree_name, opt = input$dtree_opt),
-  update_report(inp_main = list(yl = dtree_name, opt = input$dtree_opt),
+  inp <- list(yl = dtree_name)
+  if (input$dtree_opt == "min") inp$opt <- "min"
+
+  update_report(inp_main = inp,
                 fun_name = "dtree",
                 inp_out = inp_out,
                 outputs = outputs,
