@@ -18,7 +18,9 @@ ebin_inputs <- reactive({
 # Evaluate model evalbin
 ###############################################################
 output$ui_ebin_rvar <- renderUI({
-  vars <- two_level_vars()
+  withProgress(message = "Acquiring variable information", value = 1, {
+    vars <- two_level_vars()
+  })
   selectInput(inputId = "ebin_rvar", label = "Response variable:", choices = vars,
     selected = state_single("ebin_rvar", vars), multiple = FALSE)
 })
@@ -43,11 +45,11 @@ output$ui_ebin_pred <- renderUI({
 })
 
 output$ui_ebin_train <- renderUI({
-  if (is.null(input$show_filter) || input$show_filter == "FALSE" ||
-      is_empty(input$data_filter)) {
-    ebin_train <- ebin_train[1]
-    r_state$ebin_train <<- ebin_train
-  }
+  # if (is.null(input$show_filter) || input$show_filter == "FALSE" ||
+  #     is_empty(input$data_filter)) {
+  #   ebin_train <- ebin_train[1]
+  #   r_state$ebin_train <<- ebin_train
+  # }
 
   radioButtons("ebin_train", label = "Show results for:", ebin_train,
     selected = state_init("ebin_train", "All"),
@@ -61,22 +63,18 @@ output$ui_evalbin <- renderUI({
       actionButton("ebin_run", "Evaluate", width = "100%")
     ),
   	wellPanel(
-      # checkboxInput("ebin_pause", "Pause evaluation", state_init("ebin_pause", FALSE)),
 	    uiOutput("ui_ebin_rvar"),
       uiOutput("ui_ebin_lev"),
       uiOutput("ui_ebin_pred"),
       conditionalPanel("input.tabs_evalbin != 'Confusion'",
         numericInput("ebin_qnt", label = "# quantiles:",
           value = state_init("ebin_qnt", 10), min = 2)
-      # radioButtons("ebin_method", label = "Method:", ebin_method,
-        #   selected = state_init("ebin_method", "xtile"),
-        #   inline = TRUE),
       ),
       tags$table(
-        tags$td(numericInput("ebin_margin", label = "Margin:",
-          value = state_init("ebin_margin",2), width = "117px")),
         tags$td(numericInput("ebin_cost", label = "Cost:",
-          value = state_init("ebin_cost",1)))
+          value = state_init("ebin_cost",1))),
+        tags$td(numericInput("ebin_margin", label = "Margin:",
+          value = state_init("ebin_margin",2), width = "117px"))
       ),
       uiOutput("ui_ebin_train"),
       conditionalPanel("input.tabs_evalbin == 'Plot'",
@@ -85,18 +83,18 @@ output$ui_evalbin <- renderUI({
           inline = TRUE)
       ),
       conditionalPanel("input.tabs_evalbin == 'Confusion'",
-        checkboxInput("ebin_scale_y", "Scale free Y-axis", state_init("ebin_scale_y", FALSE))
+        checkboxInput("ebin_scale_y", "Scale free Y-axis", state_init("ebin_scale_y", TRUE))
       )
   	),
     conditionalPanel("input.tabs_evalbin != 'Confusion'",
-      help_and_report(modal_title = "Model evaluate classification",
+      help_and_report(modal_title = "Evaluate classification",
                       fun_name = "evalbin",
                       help_file = inclMD(file.path(getOption("radiant.path.model"),"app/tools/help/evalbin.md")))
     ),
     conditionalPanel("input.tabs_evalbin == 'Confusion'",
       help_and_report(modal_title = "Model evaluate confusion matrix",
                       fun_name = "confusion",
-                      help_file = inclMD(file.path(getOption("radiant.path.model"),"app/tools/help/confusion.md")))
+                      help_file = inclMD(file.path(getOption("radiant.path.model"),"app/tools/help/evalbin.md")))
     )
 	)
 })
@@ -115,13 +113,18 @@ confusion_plot_height <- function() 800
 # output is called from the main radiant ui.R
 output$evalbin <- renderUI({
 	register_print_output("summary_evalbin", ".summary_evalbin")
-  register_print_output("summary_confusion", ".summary_confusion")
 	register_plot_output("plot_evalbin", ".plot_evalbin",
                        	width_fun = "ebin_plot_width",
                        	height_fun = "ebin_plot_height")
+  register_print_output("summary_confusion", ".summary_confusion")
   register_plot_output("plot_confusion", ".plot_confusion",
                         width_fun = "confusion_plot_width",
                         height_fun = "confusion_plot_height")
+  register_print_output("summary_performance", ".summary_performance")
+  # register_plot_output("plot_performance", ".plot_performance",
+  #                       width_fun = "confusion_plot_width",
+  #                       height_fun = "confusion_plot_height")
+
 
 	# one output with components stacked
 	# ebin_output_panels <- tagList(
@@ -135,6 +138,12 @@ output$evalbin <- renderUI({
       plot_downloader("evalbin", height = ebin_plot_height()),
       plotOutput("plot_evalbin", height = "100%")
     ),
+    # tabPanel("Performance",
+    #    downloadLink("dl_performance_tab", "", class = "fa fa-download alignright"), br(),
+    #    verbatimTextOutput("summary_performance"),
+    #    plot_downloader("performance", height = ebin_plot_height()),
+    #    plotOutput("plot_performance", height = "100%")
+    # ),
     tabPanel("Confusion",
        downloadLink("dl_confusion_tab", "", class = "fa fa-download alignright"), br(),
        verbatimTextOutput("summary_confusion"),
@@ -191,12 +200,14 @@ output$evalbin <- renderUI({
   if (not_pressed(input$ebin_run)) return(invisible())
   if (not_available(input$ebin_rvar) || not_available(input$ebin_pred)) return(" ")
   req(input$ebin_train, !is_not(input$ebin_scale_y))
-  plot(.confusion(), scale_y = input$ebin_scale_y, shiny = TRUE)
+  plot(.confusion(), scale_y = input$ebin_scale_y)
 })
 
 observeEvent(input$evalbin_report, {
+  if (is_empty(input$ebin_rvar) || is_empty(input$ebin_pred)) return(invisible())
+  
   if (length(input$ebin_plots) > 0) {
-    inp_out <- list(plots = input$ebin_plots) %>% list("",.)
+    inp_out <- list("", list(plots = input$ebin_plots, custom = FALSE))
     outputs <- c("summary","plot")
     figs <- TRUE
   } else {
@@ -214,7 +225,11 @@ observeEvent(input$evalbin_report, {
 })
 
 observeEvent(input$confusion_report, {
-  inp_out <- list(scale_y = input$ebin_scale_y) %>% list("",.)
+  if (is_empty(input$ebin_rvar) || is_empty(input$ebin_pred)) return(invisible())
+
+  inp_out <- list("","")
+  if (!input$ebin_scale_y)
+    inp_out[[2]] <- list(scale_y = input$ebin_scale_y)
   outputs <- c("summary","plot")
   update_report(inp_main = clean_args(ebin_inputs(), ebin_args),
                 fun_name = "confusion",
@@ -229,17 +244,15 @@ observeEvent(input$confusion_report, {
 output$dl_ebin_tab <- downloadHandler(
   filename = function() { "evalbin.csv" },
   content = function(file) {
-    do.call(summary, c(list(object = .evalbin()), ebin_inputs(),
-            list(prn = FALSE))) %>%
-      write.csv(., file = file, row.names = FALSE)
+    eb <- .evalbin()
+    if (!is_empty(eb$dat)) write.csv(eb$dat, file = file, row.names = FALSE)
   }
 )
 
 output$dl_confusion_tab <- downloadHandler(
   filename = function() { "confusion.csv" },
   content = function(file) {
-    do.call(summary, c(list(object = .confusion()), ebin_inputs(),
-            list(prn = FALSE))) %>%
-      write.csv(., file = file, row.names = FALSE)
+    .confusion()$dat %>%
+    write.csv(., file = file, row.names = FALSE)
   }
 )
